package net.weasel.Vegetation;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Random;

import org.bukkit.World;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.scheduler.BukkitScheduler;

public class Vegetation extends JavaPlugin
{
	// Plugin stuff..
	public static String pluginName = "Vegetation";
	public static String pluginVersion = "2.0";
	public static boolean debugging = false;

	// System stuff
	public static Random generator = new Random();
	public static BukkitScheduler timer;
	public static ArrayList<Player> playerList = new ArrayList<Player>();

	// Timer task stuff
	public static Integer tTask;
	public static Integer timerTick = 0;
	public static Integer plantTicks = 0;
	public static Integer grassTicks = 0;
	public static Integer mossTicks = 0;
	public static Integer grazeTicks = 0;
	public static Player currentPlayer = null;
	public static Integer playerIndex = 0;
	public static Integer lastCycle = 0;
	public static Integer overGrowTicks = 0;
	
	// Growth-related stuff
	public static float grassPercent = 40;
	public static float plantsPercent = 1;
	public static float mossPercent = 1;
	public static float grazePercent = 10;
	public static Integer growthRange = 0;
	public static Integer verticalRadius = 5;

	// On/off switches
	public static boolean enableGrass = true;
	public static boolean enablePlants = true;
	public static boolean enablePumpkins = true;
	public static boolean enableFlowers = true;
	public static boolean enableFungi = true;
	public static boolean enableCacti = true;
	public static boolean enableCanes = true;
	public static boolean enableMoss = true;
	public static boolean enableGrazers = true;

	// Grazing-related stuff..
	public static boolean grazingSheep = true;
	public static boolean grazingCows = true;
	public static boolean grazingPigs = true;
	public static boolean grazingChickens = true;
	
	// Biome-related stuff..
	public static boolean growForestBiome = true;
	public static boolean growRainforestBiome = true;
	public static boolean growShrublandBiome = true;
	public static boolean growSavannahBiome = true;
	public static boolean growPlainsBiome = true;
	public static boolean growSeasonalForestBiome = true;
	public static boolean growIceDesertBiome = true;
	public static boolean growDesertBiome = true;
	public static boolean growSwamplandBiome = true;
	public static boolean growTaigaBiome = true;
	public static boolean growTundraBiome = true;
	
	public static Player overGrower = null;
	
	public String pluginIni = "plugins/Vegetation/Settings.ini";
	
	public static Vegetation plugin;

	@Override
    public void onEnable() 
    {
		plugin = this;
		
		if( new File("plugins/Vegetation/").exists() == false )
		{
			logOutput( "Settings folder does not exist - creating it.. ");
			
			boolean chk = new File("plugins/Vegetation/").mkdir();
			
			if( chk )
				logOutput( "Successfully created folder." );
			else
				logOutput( "Unable to create folder!" );
		}	
		
		if( new File(pluginIni).exists() == false )
		{
			logOutput( "Settings file does not exist - creating a new one.. ");
			
			if( createIniFile() == true )
				logOutput( "Created successfully." );
			else
				logOutput( "Unable to create ini file!" );
		}

		debugging = new File("plugins/Vegetation/debug.txt").exists();
		if( debugging ) logOutput( "debugging is enabled." );
		
        logOutput( pluginName + " v" + pluginVersion + " enabled." );
        
        timer = this.getServer().getScheduler();

        enableGrass = getBooleanSetting( "enableGrass", true );
        enablePlants = getBooleanSetting( "enablePlants", true );
    	enablePumpkins = getBooleanSetting( "enablePumpkins", true );
    	enableFlowers = getBooleanSetting( "enableFlowers", true );
    	enableFungi = getBooleanSetting( "enableFungi", true );
    	enableCacti = getBooleanSetting( "enableCacti", true );
    	enableCanes = getBooleanSetting( "enableCanes", true );
    	enableMoss = getBooleanSetting( "enableMoss", true );
    	enableGrazers = getBooleanSetting( "enableGrazers", true );

    	grazingSheep = getBooleanSetting( "grazingSheep", true );
    	grazingCows = getBooleanSetting( "grazingCows", true );
    	grazingPigs = getBooleanSetting( "grazingPigs", true );
    	grazingChickens = getBooleanSetting( "grazingChickens", true );

    	growForestBiome = getBooleanSetting( "growForestBiome", true );
    	growRainforestBiome = getBooleanSetting( "growRainforestBiome", true );
    	growShrublandBiome = getBooleanSetting( "growShrublandBiome", true );
    	growSavannahBiome = getBooleanSetting( "growSavannahBiome", true );;
    	growPlainsBiome = getBooleanSetting( "growPlainsBiome", true );
    	growSeasonalForestBiome = getBooleanSetting( "growSeasonalForestBiome", true );
		growIceDesertBiome = getBooleanSetting( "growIceDesertBiome", true );
		growDesertBiome = getBooleanSetting( "growDesertBiome", true );
		growSwamplandBiome = getBooleanSetting( "growSwamplandBiome", true );
		growTaigaBiome = getBooleanSetting( "growTaigaBiome", true );
		growTundraBiome = getBooleanSetting( "growTundraBiome", true );

        growthRange = getIntSetting( "growthRange", 100 );
    	verticalRadius = getIntSetting( "verticalRadius", 5 );
    	grassPercent = getIntSetting( "grassPercent", 40 );
    	plantsPercent = getIntSetting( "plantsPercent", 1 );
    	mossPercent = getIntSetting( "mossPercent", 1 );
    	grazePercent = getIntSetting( "grazePercent", 10 );
    	
    	if( ( grassPercent + plantsPercent + mossPercent + grazePercent ) > 100 )
    	{
        	logOutput( "The sum of the growth percentages is > 100. Please fix it." );
            this.getPluginLoader().disablePlugin(this);
    	}
    	
        if( !enableGrass && !enableGrazers && !enableMoss && !enablePlants )
        {
        	logOutput( "No vegetation is enabled. Disabling plugin." );
            this.getPluginLoader().disablePlugin(this);
        }
        else
            tTask = setupTimerTask(10);
    }
    
    @Override
    public void onDisable() 
    {
    	timer.cancelAllTasks();
        logOutput( "Plugin disabled: "+pluginName+" version "+pluginVersion);
    }
    
    public String[] getSetting( String which, String Default )
    {
        return( getSettingValue(pluginIni, which, Default, "" ) );
    }

    public boolean getBooleanSetting( String which, boolean dValue )
    {
    	boolean retVal;
    	
    	String dString = ( dValue ? "true" : "false" );
    	
        if( getSettingValue(pluginIni, which, dString, "" )[0].equalsIgnoreCase("true") )
        	retVal = true;
        else
        	retVal = false;
        
        return( retVal );
    }
    
    public int setupTimerTask( Integer interval )
    {
    	getPlayerList();
    	Integer task = timer.scheduleSyncRepeatingTask(this, new TimerTasks(this), interval, interval );
        return task;
    }
    
    public Integer getIntSetting( String item, Integer dValue )
    {
    	Integer retVal = Integer.parseInt( getSettingValue(pluginIni, item, dValue.toString(), "" )[0]);
    	return retVal;
    }

    public Float getFloatSetting( String item, Float dValue )
    {
    	Float retVal = Float.valueOf(getSettingValue(pluginIni, item, dValue.toString(), "" )[0]);
    	return retVal;
    }

    public boolean createIniFile()
    {
    	boolean retVal = false;
    	
    	try 
    	{
			FileWriter outFile = new FileWriter(pluginIni);
			PrintWriter outP = new PrintWriter(outFile);
			
			outP.println( "growthRange=100" );
			outP.println( "verticalRadius=5" );
			outP.println( "enableGrass=true" );
			outP.println( "enablePlants=true" );
			outP.println( "enableFlowers=true" );
			outP.println( "enableFungi=true" );
			outP.println( "enablePumpkins=true" );
			outP.println( "enableCacti=true" );
			outP.println( "enableCanes=true" );
			outP.println( "enableMoss=true" );
			outP.println( "enableGrazers=true" );
			outP.println( "grazingSheep=true" );
			outP.println( "grazingCows=true" );
			outP.println( "grazingPigs=true" );
			outP.println( "grazingChickens=true" );
			outP.println( "growForestBiome=true" );
			outP.println( "growRainforestBiome=true" );
			outP.println( "growShrublandBiome=true" );
			outP.println( "growSavannahBiome=true" );
			outP.println( "growPlainsBiome=true" );
			outP.println( "growSeasonalForestBiome=true" );
			outP.println( "growIceDesertBiome=true" );
			outP.println( "growDesertBiome=true" );
			outP.println( "growSwamplandBiome=true" );
			outP.println( "growTaigaBiome=true" );
			outP.println( "growTundraBiome=true" );
			outP.println( "grassPercent=40" );
			outP.println( "plantsPercent=1" );
			outP.println( "mossPercent=1" );
			outP.println( "grazePercent=10" );
			
			outP.close();
			retVal = true;
		} 
    	catch (IOException e) 
    	{
			logOutput( "Error writing to ini file." );
			retVal = false;
			e.printStackTrace();
		}
		
		return retVal;
    }
    
	public boolean onCommand(CommandSender sender, Command command, String commandLabel, String[] args) 
    {
    	String pCommand = command.getName().toLowerCase();
    	
    	if( pCommand.equals("mow") )
    	{
    		Grass.mowTheGrass( (Player) sender );
    		sender.sendMessage( "You have cut down the grass." );
    	}
    	
    	if( pCommand.equals( "grow" )  )
    	{
    		sender.sendMessage( "Growing everything.." );
    		Grass.growGrass( (Player)sender );
    		Plants.growPlant( (Player)sender );
    		Moss.growMoss( (Player)sender );
    	}
    	if( pCommand.equals( "growgrass" )  )
    	{
    		sender.sendMessage( "Growing the grass.." );
    		Grass.growGrass( (Player)sender );
    	}
    	
    	if( pCommand.equals("growplants") )
    	{
    		sender.sendMessage( "Spreading the plants.." );
    		Plants.growPlant( (Player)sender );
    	}
    	
    	if( pCommand.equals("growmoss") )
    	{
    		sender.sendMessage( "Spreading the moss.." );
    		Moss.growMoss( (Player)sender );
    	}
    	
    	if( pCommand.equals("overgrow") )
    	{
    		if( overGrowTicks > 0 )
    		{
    			if( overGrower != (Player)sender )
    			{
    				sender.sendMessage( "Overgrowth is already happening." );
    			}
    			else
    			{
	        		sender.sendMessage( "Restarting overgrowth cycle.." );
	        		overGrower = (Player)sender;
	        		overGrowTicks = 1000;
    			}
    		}
    		else
    		{
        		sender.sendMessage( "Starting overgrowth cycle.." );
        		overGrower = (Player)sender;
        		overGrowTicks = 1000;
    		}
    	}

    	return true;
    }
	            
	public String[] getSettingValue(String fileName, String optionName, String defaultValue, String splitValue)
    {
    	Boolean gotLine; // Verification variable
    	String[] returnValue = new String[100]; // Settings max at 100 values
    	String curLine;
    	
    	gotLine = false; // We don't have the settings found yet
    	
    	if( new File(fileName).exists() == false )
    	{
    		return( new String("File not found.ZQX").split("ZQX") );
    	}
		if(splitValue.equals("")) {
			splitValue = "afs4wfa3waawfa3dogrsijkge5ssioeguhwar3awwa3rar";
		}
    	
        try {
        	// Get the line from the file
			FileInputStream fstream = new FileInputStream(fileName);
			BufferedReader in = new BufferedReader(new InputStreamReader(fstream));
			
			while(in.ready()) 
			{
				curLine = in.readLine().toString();
				if(curLine.split("=", 2)[0].equalsIgnoreCase(optionName)) {
					returnValue = new String[100];
					returnValue = curLine.split("=", 2)[1].split(splitValue);
					gotLine = true;
				}
			}
			
			in.close();
			
			// If the line does not exist, create it
			if(!gotLine) 
			{
                returnValue = defaultValue.split(splitValue);
                FileOutputStream out;
                PrintStream p;
                try {
	                out = new FileOutputStream(fileName, true);
	                p = new PrintStream( out );
	                p.println (optionName+"="+defaultValue);
	                p.close();
                } catch (Exception e) {
                	logOutput("Error writing to file");
                }
			}
		}
        catch (Exception e) {
        	logOutput("-=-");
        	logOutput("File input error: "+e.toString());
        	logOutput("File input error: "+e.getStackTrace().toString());
        	logOutput("-=-");
		}
		finally {
		}
		
		return returnValue;
    }

    public static void logOutput(String text)
    {
    	System.out.println( "[" + pluginName + "]: " + text );
    }

	public static void getPlayerList()
	{
		World world = null;

		if( debugging ) logOutput( "Updating player list.." );

		Vegetation.playerList.clear();
		
		int WC = plugin.getServer().getWorlds().size();
		int PC = 0;
		int X, Y;
		
		for( X = 0; X < WC; X++ )
		{
			world = plugin.getServer().getWorlds().get(X);
			PC = world.getPlayers().size();
			
			for( Y = 0; Y < PC; Y++ )
			{
				Vegetation.playerList.add(world.getPlayers().get(Y));
			}
		}
		
		playerIndex = 0;
	}

	public static void getNextPlayer()
	{
		try
		{
			if( Vegetation.playerIndex < Vegetation.playerList.size() )
			{   
				Vegetation.playerIndex++;
				Vegetation.currentPlayer = Vegetation.playerList.get(Vegetation.playerIndex);
			}
			else
			{
				Vegetation.playerIndex = 0;
				Vegetation.currentPlayer = Vegetation.playerList.get(Vegetation.playerIndex);
			}
		}
		catch( Exception e )
		{
			Vegetation.getPlayerList();
		}
	}
	
	public static Integer getPlayerCount()
	{
		Integer retVal = 0;
		World w = null;
		int X, Y;
		
		for( X = 0; X < plugin.getServer().getWorlds().size(); X++ )
		{
			w = plugin.getServer().getWorlds().get(X);
			
			for( Y = 0; Y < w.getPlayers().size(); Y++ )
			{
				if( w.getPlayers().get(Y).isOnline() ) retVal++;
			}
		}
		
		return retVal;
	}
}
